import { Module, Provider } from '@nestjs/common';
import { MESSAGE_PUBLISHER } from './messager.di-tokens';
import {
  MessageBrokerModule,
  MessageBrokerModuleOptions,
  MessageBrokerPublisher,
} from '@mobicoop/message-broker-module';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { SERVICE_NAME } from '@src/app.constants';

const imports = [
  MessageBrokerModule.forRootAsync({
    imports: [ConfigModule],
    inject: [ConfigService],
    useFactory: async (
      configService: ConfigService,
    ): Promise<MessageBrokerModuleOptions> => ({
      uri: configService.get<string>('MESSAGE_BROKER_URI'),
      exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE'),
      name: SERVICE_NAME,
    }),
  }),
];

const providers: Provider[] = [
  {
    provide: MESSAGE_PUBLISHER,
    useClass: MessageBrokerPublisher,
  },
];

@Module({
  imports,
  providers,
  exports: [MESSAGE_PUBLISHER],
})
export class MessagerModule {}
